/// <reference types="node" />
/**
 * The possible types that can be stored in a configuration. We support anything YAML
 * does. YAML is mostly JSON, plus Buffer, which is returned for !!binary strings.
 *
 */
export interface ConfigArray extends Array<ConfigValue> {
}
export type ConfigObject = {
    [Key in string]?: ConfigValue;
};
export type ConfigValue = string | number | boolean | null | Buffer | ConfigObject | ConfigArray;
export interface Options {
    /**
     * The environment to use. (default: the value of the NODE_ENV environment variable, or
     * 'development' if it is not set)
     */
    env?: string;
    /**
     * The base directory from which to recursively load configurations (default: a
     * directory named `config`, located in the working directory, typically the top level
     * of your project)
     */
    dir?: string;
    /**
     * If true, enable loading from JavaScript files by using require(). This eval-like
     * behavior is deprecated and potentially unsafe. (default: false)
     */
    enableJs?: boolean;
    /**
     * The prefix for environment variable names that will be merged with and override any
     * values loaded from configuration files. (default: CONFIG__).
     */
    prefix?: string;
    /**
     * Whether to load config from the `local` directory. (default: true)
     */
    loadLocalConfig?: boolean;
    /**
     * JSON configuration to use as the base configuration. This will be merged with
     * configuration loaded from files and environment variables.
     */
    json?: ConfigValue;
}
export declare class Config {
    /** the currently-active environment */
    readonly env: string;
    private readonly settings;
    /** Initialize the config system. Synchronously builds the entire config tree. */
    constructor(options?: Options);
    /** Get the complete settings tree. */
    get(): ConfigValue;
    /** Get a specific setting. A dot-separated path may be used to access nested values. */
    get<T extends ConfigValue>(key: string): T | undefined;
    /** Get the top-level key names for the settings tree. */
    keys(): string[];
    /** Get the available key names at the specified config path. */
    keys(atKey: string): string[] | undefined;
    /**
     * Get a specific setting. If it is not found, throw an Error. A dot-separated path may
     * be used to access nested values.
     *
     * @returns the requested value, if found
     * @throws if the requested value was not found
     */
    getOrFail<T extends ConfigValue>(key: string): T;
    /**
     * Get a setting known to be a string. If it is not defined or not a string, an Error is
     * thrown.
     *
     * @returns the requested value, if found and is a string
     * @throws if the requested value was not found or not a string
     */
    getString(key: string): string;
    /**
     * Get a setting known to be a number. If it is not defined or not a number, an Error is
     * thrown.
     *
     * @returns the requested value, if found and is a number
     * @throws if the requested value was not found or not a number
     */
    getNumber(key: string): number;
    /**
     * Get a setting known to be a boolean. If it is not defined or not a boolean, an Error
     * is thrown.
     *
     * @returns the requested value, if found and is a boolean
     * @throws if the requested value was not found or not a boolean
     */
    getBoolean(key: string): boolean;
    /**
     * Get a setting known to be an array. If it is not defined or not an array, an Error
     * is thrown.
     *
     * @returns the requested value, if found and is a boolean
     * @throws if the requested value was not found or not a boolean
     */
    getArray<T>(key: string): T[];
    /**
     * Get a setting known to be a Buffer. If it is not defined or not a Buffer, an Error is
     * thrown. Buffer values are only possible when using YAML, for strings defined as
     * `!!binary`.
     *
     * @returns the requested value, if found and is a Buffer
     * @throws if the requested value was not found or not a Buffer
     */
    getBuffer(key: string): Buffer;
    /**
     * Get a setting known to be a Date. If it is not defined or not a Date, an Error is
     * thrown. Date values are only possible when using YAML. Within YAML this type is known
     * as `!!timestamp`.
     *
     * @returns the requested value, if found and is a Buffer
     * @throws if the requested value was not found or not a Buffer
     */
    getDate(key: string): Date;
}
