"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Config = void 0;
const lodash_1 = require("lodash");
const path = require("path");
const loadFromEnv_1 = require("./loadFromEnv");
const loadFromFiles_1 = require("./loadFromFiles");
const MISSING_VALUE = Symbol('MISSING_VALUE');
const DEFAULT_OPTIONS = {
    env: process.env.NODE_ENV || 'development',
    dir: path.resolve(process.cwd(), 'config'),
    enableJs: false,
    prefix: 'CONFIG__',
    loadLocalConfig: true,
    json: {},
};
class Config {
    /** Initialize the config system. Synchronously builds the entire config tree. */
    constructor(options) {
        const resolvedOptions = Object.assign(Object.assign({}, DEFAULT_OPTIONS), (0, lodash_1.omitBy)(options, lodash_1.isUndefined));
        if (resolvedOptions.env === 'default' ||
            (resolvedOptions.env === 'local' && resolvedOptions.loadLocalConfig)) {
            throw new Error(`[big-config] ${resolvedOptions.env} is not a valid env name`);
        }
        this.env = resolvedOptions.env;
        this.settings = {};
        this.settings = (0, lodash_1.merge)(this.settings, resolvedOptions.json);
        const defaultDir = path.resolve(resolvedOptions.dir, 'default');
        const envDir = path.resolve(resolvedOptions.dir, this.env);
        const localDir = path.resolve(resolvedOptions.dir, 'local');
        if (resolvedOptions.enableJs) {
            console.warn('[big-config] enabling potentially unsafe parsing of .js files because the ' +
                'enableJs option is true');
        }
        this.settings = (0, lodash_1.merge)(this.settings, (0, loadFromFiles_1.default)(defaultDir, resolvedOptions.enableJs));
        const lff = (0, loadFromFiles_1.default)(envDir, resolvedOptions.enableJs);
        this.settings = (0, lodash_1.merge)(this.settings, lff);
        if (resolvedOptions.loadLocalConfig) {
            this.settings = (0, lodash_1.merge)(this.settings, (0, loadFromFiles_1.default)(localDir, resolvedOptions.enableJs));
        }
        this.settings = (0, lodash_1.merge)(this.settings, (0, loadFromEnv_1.default)(resolvedOptions.prefix));
    }
    get(key) {
        if (typeof key !== 'string') {
            return (0, lodash_1.cloneDeep)(this.settings);
        }
        return (0, lodash_1.cloneDeep)((0, lodash_1.get)(this.settings, key));
    }
    keys(atKey) {
        const root = atKey
            ? (0, lodash_1.get)(this.settings, atKey)
            : this.settings;
        if (!(0, lodash_1.isPlainObject)(root)) {
            return undefined;
        }
        return Object.keys(root);
    }
    /**
     * Get a specific setting. If it is not found, throw an Error. A dot-separated path may
     * be used to access nested values.
     *
     * @returns the requested value, if found
     * @throws if the requested value was not found
     */
    getOrFail(key) {
        const value = (0, lodash_1.get)(this.settings, key, MISSING_VALUE);
        if (value === MISSING_VALUE) {
            throw new Error(`[big-config] value not found for key ${key}`);
        }
        return value;
    }
    /**
     * Get a setting known to be a string. If it is not defined or not a string, an Error is
     * thrown.
     *
     * @returns the requested value, if found and is a string
     * @throws if the requested value was not found or not a string
     */
    getString(key) {
        const value = this.getOrFail(key);
        if (typeof value !== 'string') {
            throw new Error(`[big-config] value for key ${key} is not a string`);
        }
        return value;
    }
    /**
     * Get a setting known to be a number. If it is not defined or not a number, an Error is
     * thrown.
     *
     * @returns the requested value, if found and is a number
     * @throws if the requested value was not found or not a number
     */
    getNumber(key) {
        const value = this.getOrFail(key);
        if (typeof value !== 'number') {
            throw new Error(`[big-config] value for key ${key} is not a number`);
        }
        return value;
    }
    /**
     * Get a setting known to be a boolean. If it is not defined or not a boolean, an Error
     * is thrown.
     *
     * @returns the requested value, if found and is a boolean
     * @throws if the requested value was not found or not a boolean
     */
    getBoolean(key) {
        const value = this.getOrFail(key);
        if (typeof value !== 'boolean') {
            throw new Error(`[big-config] value for key ${key} is not a boolean`);
        }
        return value;
    }
    /**
     * Get a setting known to be an array. If it is not defined or not an array, an Error
     * is thrown.
     *
     * @returns the requested value, if found and is a boolean
     * @throws if the requested value was not found or not a boolean
     */
    getArray(key) {
        const value = this.getOrFail(key);
        if (!Array.isArray(value)) {
            throw new Error(`[big-config] value for key ${key} is not an array`);
        }
        return value;
    }
    /**
     * Get a setting known to be a Buffer. If it is not defined or not a Buffer, an Error is
     * thrown. Buffer values are only possible when using YAML, for strings defined as
     * `!!binary`.
     *
     * @returns the requested value, if found and is a Buffer
     * @throws if the requested value was not found or not a Buffer
     */
    getBuffer(key) {
        const value = this.getOrFail(key);
        if (Buffer.isBuffer(value)) {
            return value;
        }
        if (value instanceof Uint8Array) {
            return Buffer.from(value.buffer);
        }
        throw new Error(`[big-config] value for key ${key} is not a Buffer or Uint8Array`);
    }
    /**
     * Get a setting known to be a Date. If it is not defined or not a Date, an Error is
     * thrown. Date values are only possible when using YAML. Within YAML this type is known
     * as `!!timestamp`.
     *
     * @returns the requested value, if found and is a Buffer
     * @throws if the requested value was not found or not a Buffer
     */
    getDate(key) {
        const value = this.getOrFail(key);
        if (!(value instanceof Date)) {
            throw new Error(`[big-config] value for key ${key} is not a Date`);
        }
        return value;
    }
}
exports.Config = Config;
//# sourceMappingURL=config.js.map