"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const assert_1 = require("assert");
const fs = require("fs");
const yaml = require("js-yaml");
const JSON5 = require("json5");
const lodash_1 = require("lodash");
const path = require("path");
/**
 * Load configuration settings from a directory containing configuration files.
 *
 * @param dir the directory from which to recursively load configurations
 * @param enableJs if true, enable loading from JavaScript files by using require(). This
 *  eval-like behavior is deprecated and potentially unsafe.
 */
function loadFromFiles(dir, enableJs = false) {
    const results = {};
    if (!fs.existsSync(dir)) {
        return results;
    }
    if (!fs.statSync(dir).isDirectory()) {
        throw new Error(`[big-config] the specified config path is not a directory: ${dir}`);
    }
    const filenames = fs
        .readdirSync(dir, { withFileTypes: true })
        .filter((x) => x.isFile())
        .sort((a, b) => a.name.localeCompare(b.name));
    // keep track of which config files we have seen, so we can detect duplicates
    const seen = {};
    filenames.forEach((dirent) => {
        const basename = path.basename(dirent.name, path.extname(dirent.name));
        const fullPath = path.resolve(dir, dirent.name);
        const ext = path.extname(fullPath);
        if (seen[basename]) {
            seen[basename].push(dirent.name);
        }
        else {
            seen[basename] = [dirent.name];
        }
        try {
            switch (ext) {
                case '.json5':
                case '.jsonc':
                    {
                        const input = fs.readFileSync(fullPath, 'utf8');
                        results[basename] = JSON5.parse(input);
                    }
                    break;
                case '.json':
                    {
                        const input = fs.readFileSync(fullPath, 'utf8');
                        results[basename] = JSON.parse(input);
                    }
                    break;
                case '.yml':
                case '.yaml':
                    {
                        const input = fs.readFileSync(fullPath, 'utf8');
                        results[basename] = yaml.load(input);
                    }
                    break;
                case '.js':
                    if (enableJs) {
                        // eslint-disable-next-line
                        results[basename] = (0, lodash_1.cloneDeep)(require(fullPath));
                    }
                    break;
            }
        }
        catch (err) {
            (0, assert_1.strict)(err instanceof Error);
            err.message = `during import of ${fullPath}: ${err.message}`;
            throw err;
        }
    });
    // warn about any duplicates
    for (const [, duplicates] of Object.entries(seen)) {
        if (duplicates.length > 1) {
            const msg = '[big-config] there are multiple configuration files in one directory with the ' +
                'same basename; this is not recommended, as their settings may conflict';
            console.warn(msg, { dir, duplicates });
        }
    }
    return results;
}
exports.default = loadFromFiles;
//# sourceMappingURL=loadFromFiles.js.map